#!/bin/sh
#================================================================
# Copyright (C) 2008 QNAP Systems, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#----------------------------------------------------------------
#
# install.sh
#
#	Abstract: 
#		A program of QPKG installation on QNAP TS-509
#		mldonkey installation
#
#	HISTORY:
#		2008/05/21	-	Created	- KenChen 
# 
#================================================================

MD_DEVIVES="md9 md13 md0 md1 md2"
SD_DEVIVES="sda sdb sdc sdd sde"
Do_Log=0
MAXRUN=100

_instr()
{
	/bin/echo "$1" | /bin/grep "$2$" | grep "block" 1>>/dev/null 2>>/dev/null
	[ $? -eq 0 ] && return 0
	/bin/echo "$1" | /bin/grep "$2$" | grep "dirtied inode" 1>>/dev/null 2>>/dev/null
	[ $? -eq 0 ] && return 0
	return 1
}

_instr_ex()
{
	/bin/echo "$1" | /bin/grep "${2}[1-4]$" | grep "block" 1>>/dev/null 2>>/dev/null
	[ $? -eq 0 ] && return 0
	/bin/echo "$1" | /bin/grep "${2}[1-4]$" | grep "dirtied inode" 1>>/dev/null 2>>/dev/null
	[ $? -eq 0 ] && return 0
	return 1
}

read_data()
{
[ -f $1 ] || return -1
[ -f $2 ] || return -1

while read LINE
do
	for i in $MD_DEVIVES
	do
		_instr "$LINE" "$i"
		if [ $? -eq 0 ]; then
			/bin/echo "$LINE" >> $2
			echo "$LINE"
		fi
	done
	
	for i in $SD_DEVIVES
	do
		_instr_ex "$LINE" "$i"
		if [ $? -eq 0 ]; then
			/bin/echo "$LINE" >> $2
			echo "$LINE"
		fi
	done
done <"$1"
}

#------------------------------------------------#
#	Main
#------------------------------------------------#
/bin/echo "---blkdevMonitor---"
case "$1" in
	-h)
	/bin/echo "Usage:"
	/bin/echo "  $0 [N]"
	/bin/echo "  N means that monitor block devices N times and generate N log files.(Default N = 100)"
	exit 0
	;;
esac

#flush
/bin/sync
/bin/echo "Countdown:"

cntdown=10
while [ ${cntdown} -gt 0 ]
do
/bin/sleep 1
/bin/echo ${cntdown}
cntdown=$[$cntdown-1]
done
/bin/echo "Turn on block_dump"
/bin/echo 1 > /proc/sys/vm/block_dump
/bin/echo "Clean old dmesg"
/bin/dmesg -c >>/dev/null

/bin/echo "Start..."

[ "x$1" != "x" ] && MAXRUN=$1
cnt=0
while [ ${cnt} -lt ${MAXRUN} ]
do

for i in $SD_DEVIVES
do
	/sbin/hdparm -y /dev/${i}
done
/bin/sleep 3

for i in $SD_DEVIVES
do
	/sbin/hdparm -C /dev/${i} 2>>/dev/null | /bin/grep active
	if [ $? -eq 0 ]; then
		Do_Log=1
		break;		
	fi
done

if [ $Do_Log -eq 1 ]; then
	TMPFILE=`/bin/mktemp '/root/temp.XXXXXX'`
	/bin/dmesg -c > "${TMPFILE}"
	CURTIME=`/bin/date '+%Y-%m-%d_%H:%M:%S'`
	OUTFILE="${CURTIME}"
	/bin/echo "[blkdevMon Logs]" > "$OUTFILE"
	read_data "$TMPFILE" "$OUTFILE"
	/bin/echo "---counter=${cnt}---" > "$OUTFILE"
	/bin/echo "---counter=${cnt}---"
	/bin/rm -f $TMPFILE
fi

/bin/echo "Countdown:"

cntdown=10
while [ ${cntdown} -gt 0 ]
do
/bin/sleep 1
/bin/echo ${cntdown}
cntdown=$[$cntdown-1]
done

cnt=$[$cnt+1]
done

/bin/echo "Turn off block_dump"
/bin/echo 0 > /proc/sys/vm/block_dump

